#include <TTCN3.hh>

#include "UpperTesterMbrPort.hh"

#include "loggers.hh"

#include "uppertester_mbr_layer.hh"
#include "uppertester_mbr_layer_factory.hh"

//=============================================================================
namespace LibItsMbr__TestSystem {

  UpperTesterMbrPort::UpperTesterMbrPort(const char *par_port_name)
    : UpperTesterMbrPort_BASE(par_port_name), _cfg_params(), _layer_params(), _layer(NULL), _time_key("UpperTesterMbrPort_Mbr::outgoing_send") {
    loggers::get_instance().log("UpperTesterMbrPort_Mbr::UpperTesterMbrPort_Mbr");
  }

  UpperTesterMbrPort::~UpperTesterMbrPort() {
    if (_layer != NULL) {
      delete _layer;
    }
  }

  void UpperTesterMbrPort::set_parameter(const char *parameter_name, const char *parameter_value) {
    loggers::get_instance().log("UpperTesterMbrPort_Mbr::set_parameter: %s=%s", parameter_name, parameter_value);
    _cfg_params.insert(std::pair<std::string, std::string>(std::string(parameter_name), std::string(parameter_value)));
    _cfg_params.log();
  }

  /*void UpperTesterMbrPort::Handle_Fd_Event(int fd, boolean is_readable,
    boolean is_writable, boolean is_error) {}*/

  void UpperTesterMbrPort::Handle_Fd_Event_Error(int /*fd*/) {}

  void UpperTesterMbrPort::Handle_Fd_Event_Writable(int /*fd*/) {}

  void UpperTesterMbrPort::Handle_Fd_Event_Readable(int /*fd*/) {}

  /*void UpperTesterMbrPort::Handle_Timeout(double time_since_last_call) {}*/

  void UpperTesterMbrPort::user_map(const char *system_port) {
    loggers::get_instance().log(">>> UpperTesterMbrPort_Mbr::user_map: %s", system_port);
    // Build layer stack
    params_its::iterator it = _cfg_params.find(std::string("params"));
    if (it != _cfg_params.end()) {
      loggers::get_instance().log("UpperTesterMbrPort_Mbr::user_map: %s", it->second.c_str());
      _layer = layer_stack_builder::get_instance()->create_layer_stack(it->second.c_str());
      if (static_cast<uppertester_mbr_layer *>(_layer) == NULL) {
        loggers::get_instance().error("UpperTesterMbrPort_Mbr::user_map: Invalid stack configuration: %s", it->second.c_str());
      }
      static_cast<uppertester_mbr_layer *>(_layer)->add_upper_port(this);
    } else {
      loggers::get_instance().error("UpperTesterMbrPort_Mbr::user_map: No layers defined in configuration file");
    }
  }

  void UpperTesterMbrPort::user_unmap(const char *system_port) {
    loggers::get_instance().log(">>> UpperTesterMbrPort_Mbr::user_unmap: %s", system_port);
    if (_layer != NULL) {
      delete _layer;
      _layer = NULL;
    }
  }

  void UpperTesterMbrPort::user_start() {}

  void UpperTesterMbrPort::user_stop() {}

  void UpperTesterMbrPort::outgoing_send(const LibItsMbr__TypesAndValues::UtMbrInitialize &send_par) {
    loggers::get_instance().log_msg(">>> UppertesterPort::outgoing_send: ", send_par);

    float duration;
    loggers::get_instance().set_start_time(_time_key);
    static_cast<uppertester_mbr_layer *>(_layer)->sendMsg(send_par, _layer_params);
    loggers::get_instance().set_stop_time(_time_key, duration);
  }

  void UpperTesterMbrPort::outgoing_send(const LibItsMbr__TypesAndValues::UtMbrTrigger &send_par) {
    loggers::get_instance().log_msg(">>> UppertesterPort::outgoing_send: ", send_par);

    float duration;
    loggers::get_instance().set_start_time(_time_key);
    static_cast<uppertester_mbr_layer *>(_layer)->sendMsg(send_par, _layer_params);
    loggers::get_instance().set_stop_time(_time_key, duration);
  }

  void UpperTesterMbrPort::receiveMsg(const Base_Type &p_ind, const params &p_params) {
    loggers::get_instance().log_msg(">>> UpperTesterMbrPort_Mbr::receive_msg: ", p_ind);
    // Sanity check
    if (!p_ind.is_bound()) {
      return;
    }

    if (std::string(p_ind.get_descriptor()->name).compare("@LibItsMbr_TypesAndValues.UtMbrResults") == 0) { // TODO To be refined
      incoming_message(static_cast<const LibItsMbr__TypesAndValues::UtMbrResults &>(p_ind));
    } else if (std::string(p_ind.get_descriptor()->name).compare("@LibItsMbr_TypesAndValues.UtMbrEventInd") == 0) { // TODO To be refined
      incoming_message(static_cast<const LibItsMbr__TypesAndValues::UtMbrEventInd &>(p_ind));
    } else {
      loggers::get_instance().warning("UpperTesterMbrPort_Mbr::receive_msg: Message not processed: %s", p_ind.get_descriptor()->name);
    }
  }

} // namespace LibItsMbr__TestSystem
