/*!
 * \file      uppertester_mbr_layer.hh
 * \brief     Header file for ITS UpperTester PKI protocol layer definition.
 * \author    ETSI TTF T027
 * \copyright ETSI Copyright Notification
 *            No part may be reproduced except as authorized by written permission.
 *            The copyright and the foregoing restriction extend to reproduction in all media.
 *            All rights reserved.
 * \version   0.1
 */
#pragma once

#include <memory>

#include "t_layer.hh"
#include "uppertester_mbr_codec.hh"

namespace LibItsMbr__TestSystem {
  class UpperTesterMbrPort;
}

namespace LibItsMbr__TypesAndValues {
  class UtMbrInitialize;
  class UtMbrTrigger;
} // namespace LibItsMbr__TypesAndValues

class uppertester_mbr_layer : public t_layer<LibItsMbr__TestSystem::UpperTesterMbrPort> {
  params                _params;
  uppertester_mbr_codec _codec;

public:
  uppertester_mbr_layer() : t_layer<LibItsMbr__TestSystem::UpperTesterMbrPort>(), _params(), _codec(){};
  uppertester_mbr_layer(const std::string &p_type, const std::string &param);
  virtual ~uppertester_mbr_layer(){};

  void sendMsg(const LibItsMbr__TypesAndValues::UtMbrInitialize &send_par, params& p_params);
  void sendMsg(const LibItsMbr__TypesAndValues::UtMbrTrigger &send_par, params& p_params);
  void sendMsg(const LibItsMbr__TypesAndValues::UtMbrEventInd &send_par, params& p_params);

  virtual void send_data(OCTETSTRING &data, params& p_params);
  virtual void receive_data(OCTETSTRING &data, params& params);
}; // End of class uppertester_mbr_layer
